@@ -26,16 +26,14 @@
 
 #include "config.h"
 
-#include "gdkscreen.h"
-#include "gdkdisplay.h"
 #include "gdkdisplaymanager.h"
 
+#include "gdkscreen.h"
+#include "gdkdisplay.h"
 #include "gdkinternals.h"
 #include "gdkmarshalers.h"
-
 #include "gdkintl.h"
 
-#include "gdkalias.h"
 
 struct _GdkDisplayManager
 {
@@ -248,8 +246,9 @@ gdk_display_manager_set_default_display (GdkDisplayManager *display_manager,
  *
  * List all currently open displays.
  * 
- * Return value: a newly allocated #GSList of #GdkDisplay objects.
- *  Free this list with g_slist_free() when you are done with it.
+ * Return value: (transfer container) (element-type GdkDisplay): a newly allocated
+ * #GSList of #GdkDisplay objects. Free this list with g_slist_free() when you
+ * are done with it.
  *
  * Since: 2.2
  **/
@@ -258,6 +257,3 @@ gdk_display_manager_list_displays (GdkDisplayManager *display_manager)
 {
   return g_slist_copy (_gdk_displays);
 }
-
-#define __GDK_DISPLAY_MANAGER_C__
-#include "gdkaliasdef.c"
