@@ -25,12 +25,11 @@
  */
 
 #include "config.h"
-#include <string.h>		/* For memset() */
-#include <math.h>
 
-#include "gdk.h"
 #include "gdkinternals.h"
 
+#include <string.h>
+#include <math.h>
 
 typedef struct _GdkIOClosure GdkIOClosure;
 
@@ -319,7 +318,7 @@ gdk_event_put (const GdkEvent *event)
   g_return_if_fail (event != NULL);
 
   if (event->any.window)
-    display = gdk_drawable_get_display (event->any.window);
+    display = gdk_window_get_display (event->any.window);
   else
     {
       GDK_NOTE (MULTIHEAD,
@@ -540,7 +539,8 @@ gdk_event_free (GdkEvent *event)
     case GDK_DRAG_STATUS:
     case GDK_DROP_START:
     case GDK_DROP_FINISHED:
-      g_object_unref (event->dnd.context);
+      if (event->dnd.context != NULL)
+        g_object_unref (event->dnd.context);
       break;
 
     case GDK_BUTTON_PRESS:
@@ -972,7 +972,7 @@ gdk_event_set_device (GdkEvent  *event,
  * If the event contains a "device" field, this function will return
  * it, else it will return %NULL.
  *
- * Returns: a #GdkDevice, or %NULL.
+ * Returns: (transfer none): a #GdkDevice, or %NULL.
  *
  * Since: 3.0
  **/
@@ -1032,20 +1032,22 @@ gdk_event_get_device (const GdkEvent *event)
     case GDK_KEY_RELEASE:
       {
         GdkDisplay *display;
-        GdkDevice *core_pointer;
+        GdkDeviceManager *device_manager;
+        GdkDevice *client_pointer;
 
         g_warning ("Event with type %d not holding a GdkDevice. "
                    "It is most likely synthesized outside Gdk/GTK+\n",
                    event->type);
 
-        display = gdk_drawable_get_display (event->any.window);
-        core_pointer = gdk_display_get_core_pointer (display);
+        display = gdk_window_get_display (event->any.window);
+        device_manager = gdk_display_get_device_manager (display);
+        client_pointer = gdk_device_manager_get_client_pointer (device_manager);
 
         if (event->type == GDK_KEY_PRESS ||
             event->type == GDK_KEY_RELEASE)
-          return gdk_device_get_associated_device (core_pointer);
+          return gdk_device_get_associated_device (client_pointer);
         else
-          return core_pointer;
+          return client_pointer;
       }
       break;
     default:
@@ -1087,7 +1089,7 @@ gdk_event_request_motions (const GdkEventMotion *event)
     {
       gdk_device_get_state (event->device, event->window, NULL, NULL);
       
-      display = gdk_drawable_get_display (event->window);
+      display = gdk_window_get_display (event->window);
       _gdk_display_enable_motion_hints (display, event->device);
     }
 }
@@ -1275,7 +1277,7 @@ gdk_event_get_screen (const GdkEvent *event)
     }
 
   if (event->any.window)
-    return gdk_drawable_get_screen (event->any.window);
+    return gdk_window_get_screen (event->any.window);
 
   return NULL;
 }
@@ -1446,7 +1448,7 @@ gdk_synthesize_window_state (GdkWindow     *window,
     {
     case GDK_WINDOW_TOPLEVEL:
     case GDK_WINDOW_TEMP: /* ? */
-      gdk_display_put_event (gdk_drawable_get_display (window), &temp_event);
+      gdk_display_put_event (gdk_window_get_display (window), &temp_event);
       break;
       
     case GDK_WINDOW_FOREIGN:
