@@ -118,41 +118,35 @@ static void         gtk_accel_label_get_property (GObject            *object,
 						  guint               prop_id,
 						  GValue             *value,
 						  GParamSpec         *pspec);
-static void         gtk_accel_label_destroy      (GtkObject          *object);
+static void         gtk_accel_label_destroy      (GtkWidget          *widget);
 static void         gtk_accel_label_finalize     (GObject            *object);
-static gboolean     gtk_accel_label_expose_event (GtkWidget          *widget,
-						  GdkEventExpose     *event);
+static gboolean     gtk_accel_label_draw         (GtkWidget          *widget,
+                                                  cairo_t            *cr);
 static const gchar *gtk_accel_label_get_string   (GtkAccelLabel      *accel_label);
 
 
-static void         gtk_accel_label_size_request_init (GtkSizeRequestIface *iface);
-static void         gtk_accel_label_get_width         (GtkSizeRequest      *widget,
-						       gint                *min_width,
-						       gint                *nat_width);
+static void         gtk_accel_label_get_preferred_width (GtkWidget           *widget,
+                                                         gint                *min_width,
+                                                         gint                *nat_width);
 
 #define GTK_ACCEL_LABEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_ACCEL_LABEL, GtkAccelLabelPrivate))
 
 
-static GtkSizeRequestIface *parent_size_request_iface;
-
-G_DEFINE_TYPE_WITH_CODE (GtkAccelLabel, gtk_accel_label, GTK_TYPE_LABEL,
-			 G_IMPLEMENT_INTERFACE (GTK_TYPE_SIZE_REQUEST,
-						gtk_accel_label_size_request_init))
+G_DEFINE_TYPE (GtkAccelLabel, gtk_accel_label, GTK_TYPE_LABEL)
 
 static void
 gtk_accel_label_class_init (GtkAccelLabelClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   
   gobject_class->finalize = gtk_accel_label_finalize;
   gobject_class->set_property = gtk_accel_label_set_property;
   gobject_class->get_property = gtk_accel_label_get_property;
-  
-  object_class->destroy = gtk_accel_label_destroy;
-   
-  widget_class->expose_event = gtk_accel_label_expose_event;
+
+  widget_class->draw = gtk_accel_label_draw;
+  widget_class->get_preferred_width = gtk_accel_label_get_preferred_width;
+  widget_class->destroy = gtk_accel_label_destroy;
 
   class->signal_quote1 = g_strdup ("<:");
   class->signal_quote2 = g_strdup (":>");
@@ -292,14 +286,14 @@ gtk_accel_label_new (const gchar *string)
 }
 
 static void
-gtk_accel_label_destroy (GtkObject *object)
+gtk_accel_label_destroy (GtkWidget *widget)
 {
-  GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (object);
+  GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
 
   gtk_accel_label_set_accel_widget (accel_label, NULL);
   gtk_accel_label_set_accel_closure (accel_label, NULL);
-  
-  GTK_OBJECT_CLASS (gtk_accel_label_parent_class)->destroy (object);
+
+  GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->destroy (widget);
 }
 
 static void
@@ -319,7 +313,7 @@ gtk_accel_label_finalize (GObject *object)
  * Fetches the widget monitored by this accelerator label. See
  * gtk_accel_label_set_accel_widget().
  *
- * Returns: the object monitored by the accelerator label, or %NULL.
+ * Returns: (transfer none): the object monitored by the accelerator label, or %NULL.
  **/
 GtkWidget*
 gtk_accel_label_get_accel_widget (GtkAccelLabel *accel_label)
@@ -349,22 +343,15 @@ gtk_accel_label_get_accel_width (GtkAccelLabel *accel_label)
 }
 
 static void
-gtk_accel_label_size_request_init (GtkSizeRequestIface *iface)
-{
-  parent_size_request_iface = g_type_interface_peek_parent (iface);
-  iface->get_width = gtk_accel_label_get_width;
-}
-
-static void
-gtk_accel_label_get_width (GtkSizeRequest  *widget,
-			   gint            *min_width,
-			   gint            *nat_width)
+gtk_accel_label_get_preferred_width (GtkWidget       *widget,
+                                     gint            *min_width,
+                                     gint            *nat_width)
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
   PangoLayout   *layout;
   gint           width;
 
-  parent_size_request_iface->get_width (widget, min_width, nat_width);
+  GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->get_preferred_width (widget, min_width, nat_width);
 
   layout = gtk_widget_create_pango_layout (GTK_WIDGET (widget), 
 					   gtk_accel_label_get_string (accel_label));
@@ -388,85 +375,86 @@ get_first_baseline (PangoLayout *layout)
 }
 
 static gboolean 
-gtk_accel_label_expose_event (GtkWidget      *widget,
-			      GdkEventExpose *event)
+gtk_accel_label_draw (GtkWidget *widget,
+                      cairo_t   *cr)
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
   GtkMisc *misc = GTK_MISC (accel_label);
   GtkTextDirection direction;
+  guint ac_width;
+  GtkAllocation allocation;
+  GtkRequisition requisition;
 
   direction = gtk_widget_get_direction (widget);
+  ac_width = gtk_accel_label_get_accel_width (accel_label);
+  gtk_widget_get_allocation (widget, &allocation);
+  gtk_widget_get_preferred_size (widget, &requisition, NULL);
 
-  if (gtk_widget_is_drawable (widget)) 
+  if (allocation.width >= requisition.width + ac_width)
     {
-      guint ac_width;
-      GtkAllocation allocation;
-      GtkRequisition requisition;
+      PangoLayout *label_layout;
+      PangoLayout *accel_layout;
+      GtkLabel *label = GTK_LABEL (widget);
 
-      ac_width = gtk_accel_label_get_accel_width (accel_label);
-      gtk_widget_get_allocation (widget, &allocation);
-      gtk_widget_size_request (widget, &requisition);
+      gint x;
+      gint y;
+      gint xpad;
 
-      if (allocation.width >= requisition.width + ac_width)
-	{
-	  PangoLayout *label_layout;
-	  PangoLayout *accel_layout;
-	  GtkLabel *label = GTK_LABEL (widget);
+      label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
 
-	  gint x;
-	  gint y;
-	  gint xpad;
+      cairo_save (cr);
 
-	  label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
+      /* XXX: Mad hack: We modify the label's width so it renders
+       * properly in its draw function that we chain to. */
+      if (direction == GTK_TEXT_DIR_RTL)
+        cairo_translate (cr, ac_width, 0);
+      if (gtk_label_get_ellipsize (label))
+        pango_layout_set_width (label_layout,
+                                pango_layout_get_width (label_layout) 
+                                - ac_width * PANGO_SCALE);
+      
+      allocation.width -= ac_width;
+      gtk_widget_set_allocation (widget, &allocation);
+      if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw)
+        GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget,
+                                                               cr);
+      allocation.width += ac_width;
+      gtk_widget_set_allocation (widget, &allocation);
+      if (gtk_label_get_ellipsize (label))
+        pango_layout_set_width (label_layout,
+                                pango_layout_get_width (label_layout) 
+                                + ac_width * PANGO_SCALE);
 
-	  if (direction == GTK_TEXT_DIR_RTL)
-	    allocation.x += ac_width;
-	  allocation.width -= ac_width;
-	  if (gtk_label_get_ellipsize (label))
-	    pango_layout_set_width (label_layout,
-				    pango_layout_get_width (label_layout) 
-				    - ac_width * PANGO_SCALE);
-	  
-	  if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->expose_event)
-	    GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->expose_event (widget, event);
-	  if (direction == GTK_TEXT_DIR_RTL)
-	    allocation.x -= ac_width;
-	  allocation.width += ac_width;
-	  if (gtk_label_get_ellipsize (label))
-	    pango_layout_set_width (label_layout,
-				    pango_layout_get_width (label_layout) 
-				    + ac_width * PANGO_SCALE);
+      cairo_restore (cr);
 
-	  gtk_misc_get_padding (misc, &xpad, NULL);
+      gtk_misc_get_padding (misc, &xpad, NULL);
 
-	  if (direction == GTK_TEXT_DIR_RTL)
-	    x = allocation.x + xpad;
-	  else
-	    x = allocation.x + allocation.width - xpad - ac_width;
-
-	  gtk_label_get_layout_offsets (GTK_LABEL (accel_label), NULL, &y);
-
-	  accel_layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
-
-	  y += get_first_baseline (label_layout) - get_first_baseline (accel_layout);
-
-          gtk_paint_layout (gtk_widget_get_style (widget),
-                            gtk_widget_get_window (widget),
-                            gtk_widget_get_state (widget),
-			    FALSE,
-                            &event->area,
-                            widget,
-                            "accellabel",
-                            x, y,
-                            accel_layout);                            
-
-          g_object_unref (accel_layout);
-	}
+      if (direction == GTK_TEXT_DIR_RTL)
+        x = xpad;
       else
-	{
-	  if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->expose_event)
-	    GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->expose_event (widget, event);
-	}
+        x = gtk_widget_get_allocated_width (widget) - xpad - ac_width;
+
+      gtk_label_get_layout_offsets (GTK_LABEL (accel_label), NULL, &y);
+
+      accel_layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
+
+      y += get_first_baseline (label_layout) - get_first_baseline (accel_layout) - allocation.y;
+
+      gtk_paint_layout (gtk_widget_get_style (widget),
+                        cr,
+                        gtk_widget_get_state (widget),
+                        FALSE,
+                        widget,
+                        "accellabel",
+                        x, y,
+                        accel_layout);                            
+
+      g_object_unref (accel_layout);
+    }
+  else
+    {
+      if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw)
+        GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget, cr);
     }
   
   return FALSE;
@@ -634,67 +622,67 @@ append_keyval_symbol (guint    accelerator_key,
 #ifdef GDK_WINDOWING_QUARTZ
   switch (accelerator_key)
   {
-  case GDK_Return:
+  case GDK_KEY_Return:
     /* U+21A9 LEFTWARDS ARROW WITH HOOK */
     g_string_append (gstring, "\xe2\x86\xa9");
     return TRUE;
 
-  case GDK_ISO_Enter:
+  case GDK_KEY_ISO_Enter:
     /* U+2324 UP ARROWHEAD BETWEEN TWO HORIZONTAL BARS */
     g_string_append (gstring, "\xe2\x8c\xa4");
     return TRUE;
 
-  case GDK_Left:
+  case GDK_KEY_Left:
     /* U+2190 LEFTWARDS ARROW */
     g_string_append (gstring, "\xe2\x86\x90");
     return TRUE;
 
-  case GDK_Up:
+  case GDK_KEY_Up:
     /* U+2191 UPWARDS ARROW */
     g_string_append (gstring, "\xe2\x86\x91");
     return TRUE;
 
-  case GDK_Right:
+  case GDK_KEY_Right:
     /* U+2192 RIGHTWARDS ARROW */
     g_string_append (gstring, "\xe2\x86\x92");
     return TRUE;
 
-  case GDK_Down:
+  case GDK_KEY_Down:
     /* U+2193 DOWNWARDS ARROW */
     g_string_append (gstring, "\xe2\x86\x93");
     return TRUE;
 
-  case GDK_Page_Up:
+  case GDK_KEY_Page_Up:
     /* U+21DE UPWARDS ARROW WITH DOUBLE STROKE */
     g_string_append (gstring, "\xe2\x87\x9e");
     return TRUE;
 
-  case GDK_Page_Down:
+  case GDK_KEY_Page_Down:
     /* U+21DF DOWNWARDS ARROW WITH DOUBLE STROKE */
     g_string_append (gstring, "\xe2\x87\x9f");
     return TRUE;
 
-  case GDK_Home:
+  case GDK_KEY_Home:
     /* U+2196 NORTH WEST ARROW */
     g_string_append (gstring, "\xe2\x86\x96");
     return TRUE;
 
-  case GDK_End:
+  case GDK_KEY_End:
     /* U+2198 SOUTH EAST ARROW */
     g_string_append (gstring, "\xe2\x86\x98");
     return TRUE;
 
-  case GDK_Escape:
+  case GDK_KEY_Escape:
     /* U+238B BROKEN CIRCLE WITH NORTHWEST ARROW */
     g_string_append (gstring, "\xe2\x8e\x8b");
     return TRUE;
 
-  case GDK_BackSpace:
+  case GDK_KEY_BackSpace:
     /* U+232B ERASE TO THE LEFT */
     g_string_append (gstring, "\xe2\x8c\xab");
     return TRUE;
 
-  case GDK_Delete:
+  case GDK_KEY_Delete:
     /* U+2326 ERASE TO THE RIGHT */
     g_string_append (gstring, "\xe2\x8c\xa6");
     return TRUE;
