@@ -46,7 +46,7 @@ G_BEGIN_DECLS
 
 
 typedef struct _GtkColorSelection       GtkColorSelection;
-typedef struct _ColorSelectionPrivate   ColorSelectionPrivate;
+typedef struct _GtkColorSelectionPrivate  GtkColorSelectionPrivate;
 typedef struct _GtkColorSelectionClass  GtkColorSelectionClass;
 
 typedef void (* GtkColorSelectionChangePaletteFunc) (const GdkColor    *colors,
@@ -60,7 +60,7 @@ struct _GtkColorSelection
   GtkVBox parent_instance;
 
   /* < private_data > */
-  ColorSelectionPrivate *private_data;
+  GtkColorSelectionPrivate *private_data;
 };
 
 struct _GtkColorSelectionClass
@@ -104,6 +104,15 @@ void     gtk_color_selection_get_previous_color  (GtkColorSelection *colorsel,
 						  GdkColor          *color);
 guint16  gtk_color_selection_get_previous_alpha  (GtkColorSelection *colorsel);
 
+void     gtk_color_selection_set_current_rgba    (GtkColorSelection *colorsel,
+                                                  const GdkRGBA     *rgba);
+void     gtk_color_selection_get_current_rgba    (GtkColorSelection *colorsel,
+                                                  GdkRGBA           *rgba);
+void     gtk_color_selection_set_previous_rgba   (GtkColorSelection *colorsel,
+                                                  const GdkRGBA     *rgba);
+void     gtk_color_selection_get_previous_rgba   (GtkColorSelection *colorsel,
+                                                  GdkRGBA           *rgba);
+
 gboolean gtk_color_selection_is_adjusting        (GtkColorSelection *colorsel);
 
 gboolean gtk_color_selection_palette_from_string (const gchar       *str,
