@@ -138,8 +138,7 @@ typedef enum {
 
 typedef enum {
   RELOAD_EMPTY,			/* No folder has been set */
-  RELOAD_HAS_FOLDER,		/* We have a folder, although it may not be completely loaded yet; no need to reload */
-  RELOAD_WAS_UNMAPPED		/* We had a folder but got unmapped; reload is needed */
+  RELOAD_HAS_FOLDER		/* We have a folder, although it may not be completely loaded yet; no need to reload */
 } ReloadState;
 
 typedef enum {
@@ -279,6 +278,8 @@ struct _GtkFileChooserDefault
   gint sort_column;
   GtkSortType sort_order;
 
+  GSettings *settings;
+
 #if 0
   GdkDragContext *shortcuts_drag_context;
   GSource *shortcuts_drag_outside_idle;
