@@ -1388,8 +1388,8 @@ render_icon_name_pixbuf (GtkIconSource    *icon_source,
 
   if (widget && gtk_widget_has_screen (widget))
     screen = gtk_widget_get_screen (widget);
-  else if (style && style->colormap)
-    screen = gdk_colormap_get_screen (style->colormap);
+  else if (style && style->visual)
+    screen = gdk_visual_get_screen (style->visual);
   else
     {
       screen = gdk_screen_get_default ();
@@ -1463,6 +1463,8 @@ render_icon_name_pixbuf (GtkIconSource    *icon_source,
           tmp_pixbuf = gtk_icon_info_load_icon (info, &error);
           gtk_icon_info_free (info);
         }
+      else
+        tmp_pixbuf = NULL;
     }
   else
     {
@@ -1619,7 +1621,7 @@ render_fallback_image (GtkStyle          *style,
  * (perhaps because an image file fails to load), a default "missing
  * image" icon will be returned instead.
  *
- * Return value: a #GdkPixbuf to be displayed
+ * Return value: (transfer full): a #GdkPixbuf to be displayed
  */
 GdkPixbuf*
 gtk_icon_set_render_icon (GtkIconSet        *icon_set,
@@ -2103,7 +2105,7 @@ gtk_icon_source_get_icon_name (const GtkIconSource *source)
  * virtual function. The reference count on the pixbuf is
  * not incremented.
  *
- * Return value: source pixbuf
+ * Return value: (transfer none): source pixbuf
  */
 GdkPixbuf*
 gtk_icon_source_get_pixbuf (const GtkIconSource *source)
