@@ -114,7 +114,6 @@ static void gtk_menu_item_position_menu  (GtkMenu          *menu,
 					  gboolean         *push_in,
 					  gpointer          user_data);
 static void gtk_menu_item_show_all       (GtkWidget        *widget);
-static void gtk_menu_item_hide_all       (GtkWidget        *widget);
 static void gtk_menu_item_forall         (GtkContainer    *container,
 					  gboolean         include_internals,
 					  GtkCallback      callback,
@@ -194,7 +193,6 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   widget_class->enter_notify_event = gtk_menu_item_enter;
   widget_class->leave_notify_event = gtk_menu_item_leave;
   widget_class->show_all = gtk_menu_item_show_all;
-  widget_class->hide_all = gtk_menu_item_hide_all;
   widget_class->mnemonic_activate = gtk_menu_item_mnemonic_activate;
   widget_class->parent_set = gtk_menu_item_parent_set;
   widget_class->can_activate_accel = gtk_menu_item_can_activate_accel;
@@ -1104,15 +1102,18 @@ gtk_menu_item_sync_action_properties (GtkActivatable *activatable,
     {
       label = gtk_bin_get_child (GTK_BIN (menu_item));
 
-      /* make sure label is a label */
+      /* make sure label is a label, deleting it otherwise */
       if (label && !GTK_IS_LABEL (label))
 	{
 	  gtk_container_remove (GTK_CONTAINER (menu_item), label);
 	  label = NULL;
 	}
-
+      /* Make sure that menu_item has a label and that any
+       * accelerators are set */
       gtk_menu_item_ensure_label (menu_item);
       gtk_menu_item_set_use_underline (menu_item, TRUE);
+      /* Make label point to the menu_item's label */
+      label = gtk_bin_get_child (GTK_BIN (menu_item));
 
       if (GTK_IS_ACCEL_LABEL (label) && gtk_action_get_accel_path (action))
         {
@@ -2184,23 +2185,6 @@ gtk_menu_item_show_all (GtkWidget *widget)
   gtk_widget_show (widget);
 }
 
-static void
-gtk_menu_item_hide_all (GtkWidget *widget)
-{
-  GtkMenuItem *menu_item;
-
-  g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-
-  gtk_widget_hide (widget);
-
-  menu_item = GTK_MENU_ITEM (widget);
-
-  /* hide children including submenu */
-  gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_hide_all, NULL);
-  if (menu_item->submenu)
-    gtk_widget_hide_all (menu_item->submenu);
-}
-
 static gboolean
 gtk_menu_item_can_activate_accel (GtkWidget *widget,
 				  guint      signal_id)
