@@ -31,6 +31,7 @@
 
 #include "gtkorientable.h"
 #include "gtkruler.h"
+#include "gtktypeutils.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
@@ -46,7 +47,7 @@ struct _GtkRulerPrivate
   GtkOrientation        orientation;
   GtkRulerMetric       *metric;
 
-  GdkPixmap            *backing_store;
+  cairo_surface_t      *backing_store;
 
   gint                  slider_size;
   gint                  xsrc;
@@ -79,17 +80,26 @@ static void     gtk_ruler_get_property    (GObject        *object,
                                            GParamSpec     *pspec);
 static void     gtk_ruler_realize         (GtkWidget      *widget);
 static void     gtk_ruler_unrealize       (GtkWidget      *widget);
-static void     gtk_ruler_size_request    (GtkWidget      *widget,
-                                           GtkRequisition *requisition);
+static void     gtk_ruler_get_preferred_width
+                                          (GtkWidget      *widget,
+                                           gint           *minimum,
+                                           gint           *natural);
+static void     gtk_ruler_get_preferred_height
+                                          (GtkWidget        *widget,
+                                           gint             *minimum,
+                                           gint             *natural);
 static void     gtk_ruler_size_allocate   (GtkWidget      *widget,
                                            GtkAllocation  *allocation);
 static gboolean gtk_ruler_motion_notify   (GtkWidget      *widget,
                                            GdkEventMotion *event);
-static gboolean gtk_ruler_expose          (GtkWidget      *widget,
-                                           GdkEventExpose *event);
+static gboolean gtk_ruler_draw            (GtkWidget      *widget,
+                                           cairo_t        *cr);
 static void     gtk_ruler_make_pixmap     (GtkRuler       *ruler);
-static void     gtk_ruler_real_draw_ticks (GtkRuler       *ruler);
-static void     gtk_ruler_real_draw_pos   (GtkRuler       *ruler);
+static void     gtk_ruler_draw_ticks      (GtkRuler       *ruler);
+static void     gtk_ruler_real_draw_ticks (GtkRuler       *ruler,
+                                           cairo_t        *cr);
+static void     gtk_ruler_real_draw_pos   (GtkRuler       *ruler,
+                                           cairo_t        *cr);
 
 
 static const GtkRulerMetric ruler_metrics[] =
@@ -116,17 +126,16 @@ gtk_ruler_class_init (GtkRulerClass *class)
 
   widget_class->realize = gtk_ruler_realize;
   widget_class->unrealize = gtk_ruler_unrealize;
-  widget_class->size_request = gtk_ruler_size_request;
+  widget_class->get_preferred_width = gtk_ruler_get_preferred_width;
+  widget_class->get_preferred_height = gtk_ruler_get_preferred_height;
   widget_class->size_allocate = gtk_ruler_size_allocate;
   widget_class->motion_notify_event = gtk_ruler_motion_notify;
-  widget_class->expose_event = gtk_ruler_expose;
+  widget_class->draw = gtk_ruler_draw;
 
   class->draw_ticks = gtk_ruler_real_draw_ticks;
   class->draw_pos = gtk_ruler_real_draw_pos;
 
-  g_object_class_override_property (gobject_class,
-                                    PROP_ORIENTATION,
-                                    "orientation");
+  g_object_class_override_property (gobject_class, PROP_ORIENTATION, "orientation");
 
   g_object_class_install_property (gobject_class,
                                    PROP_LOWER,
@@ -303,6 +312,26 @@ gtk_ruler_new (GtkOrientation orientation)
                        NULL);
 }
 
+/**
+ * gtk_ruler_invalidate_ticks:
+ * @ruler: the ruler to invalidate
+ *
+ * For performance reasons, #GtkRuler keeps a backbuffer containing the
+ * prerendered contents of the ticks. To cause a repaint this buffer,
+ * call this function instead of gtk_widget_queue_draw().
+ **/
+static void
+gtk_ruler_invalidate_ticks (GtkRuler *ruler)
+{
+  g_return_if_fail (GTK_IS_RULER (ruler));
+
+  if (ruler->priv->backing_store == NULL)
+    return;
+
+  gtk_ruler_draw_ticks (ruler);
+  gtk_widget_queue_draw (GTK_WIDGET (ruler));
+}
+
 void
 gtk_ruler_set_metric (GtkRuler      *ruler,
 		      GtkMetricType  metric)
@@ -315,10 +344,9 @@ gtk_ruler_set_metric (GtkRuler      *ruler,
 
   priv->metric = (GtkRulerMetric *) &ruler_metrics[metric];
 
-  if (gtk_widget_is_drawable (GTK_WIDGET (ruler)))
-    gtk_widget_queue_draw (GTK_WIDGET (ruler));
-
   g_object_notify (G_OBJECT (ruler), "metric");
+
+  gtk_ruler_invalidate_ticks (ruler);
 }
 
 /**
@@ -395,8 +423,7 @@ gtk_ruler_set_range (GtkRuler *ruler,
     }
   g_object_thaw_notify (G_OBJECT (ruler));
 
-  if (gtk_widget_is_drawable (GTK_WIDGET (ruler)))
-    gtk_widget_queue_draw (GTK_WIDGET (ruler));
+  gtk_ruler_invalidate_ticks (ruler);
 }
 
 /**
@@ -434,25 +461,25 @@ gtk_ruler_get_range (GtkRuler *ruler,
     *max_size = priv->max_size;
 }
 
-void
+static void
 gtk_ruler_draw_ticks (GtkRuler *ruler)
 {
+  GtkRulerPrivate *priv = ruler->priv;
+  cairo_t *cr;
+
   g_return_if_fail (GTK_IS_RULER (ruler));
 
+  if (priv->backing_store == NULL)
+    return;
+  
+  cr = cairo_create (priv->backing_store);
+
   if (GTK_RULER_GET_CLASS (ruler)->draw_ticks)
-    GTK_RULER_GET_CLASS (ruler)->draw_ticks (ruler);
+    GTK_RULER_GET_CLASS (ruler)->draw_ticks (ruler, cr);
+
+  cairo_destroy (cr);
 }
 
-void
-gtk_ruler_draw_pos (GtkRuler *ruler)
-{
-  g_return_if_fail (GTK_IS_RULER (ruler));
-
-  if (GTK_RULER_GET_CLASS (ruler)->draw_pos)
-     GTK_RULER_GET_CLASS (ruler)->draw_pos (ruler);
-}
-
-
 static void
 gtk_ruler_realize (GtkWidget *widget)
 {
@@ -475,13 +502,12 @@ gtk_ruler_realize (GtkWidget *widget)
   attributes.height = allocation.height;
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.visual = gtk_widget_get_visual (widget);
-  attributes.colormap = gtk_widget_get_colormap (widget);
   attributes.event_mask = gtk_widget_get_events (widget);
   attributes.event_mask |= (GDK_EXPOSURE_MASK |
 			    GDK_POINTER_MOTION_MASK |
 			    GDK_POINTER_MOTION_HINT_MASK);
 
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
   window = gdk_window_new (gtk_widget_get_parent_window (widget),
                            &attributes, attributes_mask);
@@ -503,7 +529,7 @@ gtk_ruler_unrealize (GtkWidget *widget)
 
   if (priv->backing_store)
     {
-      g_object_unref (priv->backing_store);
+      cairo_surface_destroy (priv->backing_store);
       priv->backing_store = NULL;
     }
 
@@ -511,25 +537,43 @@ gtk_ruler_unrealize (GtkWidget *widget)
 }
 
 static void
-gtk_ruler_size_request (GtkWidget      *widget,
-                        GtkRequisition *requisition)
+gtk_ruler_get_preferred_size (GtkWidget      *widget,
+                              GtkOrientation  orientation,
+                              gint           *minimum,
+                              gint           *natural)
 {
   GtkRuler *ruler = GTK_RULER (widget);
   GtkRulerPrivate *priv = ruler->priv;
   GtkStyle *style;
+  gint thickness;
 
   style = gtk_widget_get_style (widget);
 
-  if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-      requisition->width  = style->xthickness * 2 + 1;
-      requisition->height = style->ythickness * 2 + RULER_WIDTH;
-    }
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    thickness = style->xthickness;
   else
-    {
-      requisition->width  = style->xthickness * 2 + RULER_WIDTH;
-      requisition->height = style->ythickness * 2 + 1;
-    }
+    thickness = style->ythickness;
+
+  if (priv->orientation == orientation)
+    *minimum = *natural = thickness * 2 + 1;
+  else
+    *minimum = *natural = thickness * 2 + RULER_WIDTH;
+}
+
+static void
+gtk_ruler_get_preferred_width (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
+{
+  gtk_ruler_get_preferred_size (widget, GTK_ORIENTATION_HORIZONTAL, minimum, natural);
+}
+
+static void
+gtk_ruler_get_preferred_height (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
+{
+  gtk_ruler_get_preferred_size (widget, GTK_ORIENTATION_VERTICAL, minimum, natural);
 }
 
 static void
@@ -537,6 +581,12 @@ gtk_ruler_size_allocate (GtkWidget     *widget,
 			 GtkAllocation *allocation)
 {
   GtkRuler *ruler = GTK_RULER (widget);
+  GtkAllocation old_allocation;
+  gboolean resized;
+
+  gtk_widget_get_allocation (widget, &old_allocation);
+  resized = (old_allocation.width != allocation->width ||
+             old_allocation.height != allocation->height);
 
   gtk_widget_set_allocation (widget, allocation);
 
@@ -546,7 +596,8 @@ gtk_ruler_size_allocate (GtkWidget     *widget,
 			      allocation->x, allocation->y,
 			      allocation->width, allocation->height);
 
-      gtk_ruler_make_pixmap (ruler);
+      if (resized)
+        gtk_ruler_make_pixmap (ruler);
     }
 }
 
@@ -572,33 +623,23 @@ gtk_ruler_motion_notify (GtkWidget      *widget,
 
   g_object_notify (G_OBJECT (ruler), "position");
 
-  /*  Make sure the ruler has been allocated already  */
-  if (priv->backing_store != NULL)
-    gtk_ruler_draw_pos (ruler);
+  gtk_widget_queue_draw (widget);
 
   return FALSE;
 }
 
 static gboolean
-gtk_ruler_expose (GtkWidget      *widget,
-		  GdkEventExpose *event)
+gtk_ruler_draw (GtkWidget *widget,
+                cairo_t   *cr)
 {
-  if (gtk_widget_is_drawable (widget))
-    {
-      GtkRuler *ruler = GTK_RULER (widget);
-      GtkRulerPrivate *priv = ruler->priv;
-      cairo_t *cr;
+  GtkRuler *ruler = GTK_RULER (widget);
+  GtkRulerPrivate *priv = ruler->priv;
 
-      gtk_ruler_draw_ticks (ruler);
-
-      cr = gdk_cairo_create (gtk_widget_get_window (widget));
-      gdk_cairo_set_source_pixmap (cr, priv->backing_store, 0, 0);
-      gdk_cairo_rectangle (cr, &event->area);
-      cairo_fill (cr);
-      cairo_destroy (cr);
-      
-      gtk_ruler_draw_pos (ruler);
-    }
+  cairo_set_source_surface (cr, priv->backing_store, 0, 0);
+  cairo_paint (cr);
+  
+  if (GTK_RULER_GET_CLASS (ruler)->draw_pos)
+     GTK_RULER_GET_CLASS (ruler)->draw_pos (ruler, cr);
 
   return FALSE;
 }
@@ -609,41 +650,34 @@ gtk_ruler_make_pixmap (GtkRuler *ruler)
   GtkRulerPrivate *priv = ruler->priv;
   GtkAllocation allocation;
   GtkWidget *widget;
-  gint width;
-  gint height;
 
   widget = GTK_WIDGET (ruler);
 
   gtk_widget_get_allocation (widget, &allocation);
 
   if (priv->backing_store)
-    {
-      gdk_drawable_get_size (priv->backing_store, &width, &height);
-      if ((width == allocation.width) &&
-	  (height == allocation.height))
-	return;
+    cairo_surface_destroy (priv->backing_store);
 
-      g_object_unref (priv->backing_store);
-    }
-
-  priv->backing_store = gdk_pixmap_new (gtk_widget_get_window (widget),
-                                        allocation.width,
-                                        allocation.height,
-                                        -1);
+  priv->backing_store = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
+                                                           CAIRO_CONTENT_COLOR,
+                                                           allocation.width,
+                                                           allocation.height);
 
   priv->xsrc = 0;
   priv->ysrc = 0;
+
+  gtk_ruler_draw_ticks (ruler);
 }
 
 static void
-gtk_ruler_real_draw_ticks (GtkRuler *ruler)
+gtk_ruler_real_draw_ticks (GtkRuler *ruler,
+                           cairo_t  *cr)
 {
-  GtkAllocation allocation;
   GtkWidget *widget = GTK_WIDGET (ruler);
   GtkRulerPrivate *priv = ruler->priv;
   GtkStyle *style;
-  cairo_t *cr;
   gint i, j;
+  gint w, h;
   gint width, height;
   gint xthickness;
   gint ythickness;
@@ -662,11 +696,7 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
   PangoLayout *layout;
   PangoRectangle logical_rect, ink_rect;
 
-  if (!gtk_widget_is_drawable (widget))
-    return;
-
   style = gtk_widget_get_style (widget);
-  gtk_widget_get_allocation (widget, &allocation);
 
   xthickness = style->xthickness;
   ythickness = style->ythickness;
@@ -677,36 +707,38 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
   digit_height = PANGO_PIXELS (ink_rect.height) + 2;
   digit_offset = ink_rect.y;
 
+  w = gtk_widget_get_allocated_width (widget);
+  h = gtk_widget_get_allocated_height (widget);
+
   if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      width = allocation.width;
-      height = allocation.height - ythickness * 2;
+      width = w;
+      height = h - ythickness * 2;
     }
   else
     {
-      width = allocation.height;
-      height = allocation.width - ythickness * 2;
+      width = h;
+      height = w - ythickness * 2;
     }
 
 #define DETAILE(private) (priv->orientation == GTK_ORIENTATION_HORIZONTAL ? "hruler" : "vruler");
 
-  gtk_paint_box (style, priv->backing_store,
-		 GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		 NULL, widget,
-                 priv->orientation == GTK_ORIENTATION_HORIZONTAL ?
-                 "hruler" : "vruler",
-		 0, 0,
-                 allocation.width, allocation.height);
-
-  cr = gdk_cairo_create (priv->backing_store);
   gdk_cairo_set_source_color (cr, &style->fg[gtk_widget_get_state (widget)]);
 
+  gtk_paint_box (style, cr,
+                       GTK_STATE_NORMAL, GTK_SHADOW_OUT,
+                       widget,
+                       priv->orientation == GTK_ORIENTATION_HORIZONTAL ?
+                       "hruler" : "vruler",
+                       0, 0,
+                       w, h);
+
   if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       cairo_rectangle (cr,
                        xthickness,
                        height + ythickness,
-                       allocation.width - 2 * xthickness,
+                       w - 2 * xthickness,
                        1);
     }
   else
@@ -715,7 +747,7 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
                        height + xthickness,
                        ythickness,
                        1,
-                       allocation.height - 2 * ythickness);
+                       h - 2 * ythickness);
     }
 
   upper = priv->upper / priv->metric->pixels_per_unit;
@@ -802,6 +834,7 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
                                height + xthickness - length, pos,
                                length,                       1);
             }
+          cairo_fill (cr);
 
 	  /* draw label */
 	  if (i == 0)
@@ -814,14 +847,13 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
                   pango_layout_get_extents (layout, &logical_rect, NULL);
 
                   gtk_paint_layout (style,
-                                    priv->backing_store,
-                                    gtk_widget_get_state (widget),
-                                    FALSE,
-                                    NULL,
-                                    widget,
-                                    "hruler",
-                                    pos + 2, ythickness + PANGO_PIXELS (logical_rect.y - digit_offset),
-                                    layout);
+                                          cr,
+                                          gtk_widget_get_state (widget),
+                                          FALSE,
+                                          widget,
+                                          "hruler",
+                                          pos + 2, ythickness + PANGO_PIXELS (logical_rect.y - digit_offset),
+                                          layout);
                 }
               else
                 {
@@ -831,15 +863,14 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
                       pango_layout_get_extents (layout, NULL, &logical_rect);
 
                       gtk_paint_layout (style,
-                                        priv->backing_store,
-                                        gtk_widget_get_state (widget),
-                                        FALSE,
-                                        NULL,
-                                        widget,
-                                        "vruler",
-                                        xthickness + 1,
-                                        pos + digit_height * j + 2 + PANGO_PIXELS (logical_rect.y - digit_offset),
-                                        layout);
+                                              cr,
+                                              gtk_widget_get_state (widget),
+                                              FALSE,
+                                              widget,
+                                              "vruler",
+                                              xthickness + 1,
+                                              pos + digit_height * j + 2 + PANGO_PIXELS (logical_rect.y - digit_offset),
+                                              layout);
                     }
                 }
 	    }
@@ -847,108 +878,82 @@ gtk_ruler_real_draw_ticks (GtkRuler *ruler)
     }
 
   cairo_fill (cr);
-out:
-  cairo_destroy (cr);
 
+out:
   g_object_unref (layout);
 }
 
 static void
-gtk_ruler_real_draw_pos (GtkRuler *ruler)
+gtk_ruler_real_draw_pos (GtkRuler *ruler,
+                         cairo_t  *cr)
 {
-  GtkAllocation allocation;
   GtkWidget *widget = GTK_WIDGET (ruler);
   GtkRulerPrivate *priv = ruler->priv;
   GtkStyle *style;
-  gint x, y;
-  gint width, height;
+  gint x, y, width, height;
   gint bs_width, bs_height;
   gint xthickness;
   gint ythickness;
   gdouble increment;
 
-  if (gtk_widget_is_drawable (widget))
+  style = gtk_widget_get_style (widget);
+  xthickness = style->xthickness;
+  ythickness = style->ythickness;
+  width = gtk_widget_get_allocated_width (widget);
+  height = gtk_widget_get_allocated_height (widget);
+
+  if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      style = gtk_widget_get_style (widget);
-      gtk_widget_get_allocation (widget, &allocation);
+      height -= ythickness * 2;
 
-      xthickness = style->xthickness;
-      ythickness = style->ythickness;
-      width = allocation.width;
-      height = allocation.height;
+      bs_width = height / 2 + 2;
+      bs_width |= 1;  /* make sure it's odd */
+      bs_height = bs_width / 2 + 1;
+    }
+  else
+    {
+      width -= xthickness * 2;
 
+      bs_height = width / 2 + 2;
+      bs_height |= 1;  /* make sure it's odd */
+      bs_width = bs_height / 2 + 1;
+    }
+
+  if ((bs_width > 0) && (bs_height > 0))
+    {
       if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          height -= ythickness * 2;
+          increment = (gdouble) width / (priv->upper - priv->lower);
 
-          bs_width = height / 2 + 2;
-          bs_width |= 1;  /* make sure it's odd */
-          bs_height = bs_width / 2 + 1;
+          x = ROUND ((priv->position - priv->lower) * increment) + (xthickness - bs_width) / 2 - 1;
+          y = (height + bs_height) / 2 + ythickness;
         }
       else
         {
-          width -= xthickness * 2;
+          increment = (gdouble) height / (priv->upper - priv->lower);
 
-          bs_height = width / 2 + 2;
-          bs_height |= 1;  /* make sure it's odd */
-          bs_width = bs_height / 2 + 1;
+          x = (width + bs_width) / 2 + xthickness;
+          y = ROUND ((priv->position - priv->lower) * increment) + (ythickness - bs_height) / 2 - 1;
         }
 
-      if ((bs_width > 0) && (bs_height > 0))
-	{
-          GdkWindow *window;
+      gdk_cairo_set_source_color (cr, &style->fg[gtk_widget_get_state (widget)]);
 
-          window = gtk_widget_get_window (widget);
+      cairo_move_to (cr, x, y);
 
-	  cairo_t *cr = gdk_cairo_create (window);
+      if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          cairo_line_to (cr, x + bs_width / 2.0, y + bs_height);
+          cairo_line_to (cr, x + bs_width,       y);
+        }
+      else
+        {
+          cairo_line_to (cr, x + bs_width, y + bs_height / 2.0);
+          cairo_line_to (cr, x,            y + bs_height);
+        }
 
-	  /*  If a backing store exists, restore the ruler  */
-	  if (priv->backing_store) {
-            cairo_t *cr = gdk_cairo_create (window);
+      cairo_fill (cr);
 
-            gdk_cairo_set_source_pixmap (cr, priv->backing_store, 0, 0);
-            cairo_rectangle (cr, priv->xsrc, priv->ysrc, bs_width, bs_height);
-            cairo_fill (cr);
-
-            cairo_destroy (cr);
-          }
-
-          if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
-            {
-              increment = (gdouble) width / (priv->upper - priv->lower);
-
-              x = ROUND ((priv->position - priv->lower) * increment) + (xthickness - bs_width) / 2 - 1;
-              y = (height + bs_height) / 2 + ythickness;
-            }
-          else
-            {
-              increment = (gdouble) height / (priv->upper - priv->lower);
-
-              x = (width + bs_width) / 2 + xthickness;
-              y = ROUND ((priv->position - priv->lower) * increment) + (ythickness - bs_height) / 2 - 1;
-            }
-
-	  gdk_cairo_set_source_color (cr, &style->fg[gtk_widget_get_state (widget)]);
-
-	  cairo_move_to (cr, x, y);
-
-          if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
-            {
-              cairo_line_to (cr, x + bs_width / 2.0, y + bs_height);
-              cairo_line_to (cr, x + bs_width,       y);
-            }
-          else
-            {
-              cairo_line_to (cr, x + bs_width, y + bs_height / 2.0);
-              cairo_line_to (cr, x,            y + bs_height);
-            }
-
-	  cairo_fill (cr);
-
-	  cairo_destroy (cr);
-
-	  priv->xsrc = x;
-	  priv->ysrc = y;
-	}
+      priv->xsrc = x;
+      priv->ysrc = y;
     }
 }
