@@ -55,9 +55,10 @@ G_BEGIN_DECLS
 #define GTK_RULER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_RULER, GtkRulerClass))
 
 
-typedef struct _GtkRuler        GtkRuler;
-typedef struct _GtkRulerClass   GtkRulerClass;
-typedef struct _GtkRulerMetric  GtkRulerMetric;
+typedef struct _GtkRuler              GtkRuler;
+typedef struct _GtkRulerPrivate       GtkRulerPrivate;
+typedef struct _GtkRulerClass         GtkRulerClass;
+typedef struct _GtkRulerMetric        GtkRulerMetric;
 
 /* All distances below are in 1/72nd's of an inch. (According to
  * Adobe that's a point, but points are really 1/72.27 in.)
@@ -66,29 +67,18 @@ struct _GtkRuler
 {
   GtkWidget widget;
 
-  GdkPixmap *GSEAL (backing_store);
-  GdkGC *GSEAL (non_gr_exp_gc);		/* unused */
-  GtkRulerMetric *GSEAL (metric);
-  gint GSEAL (xsrc);
-  gint GSEAL (ysrc);
-  gint GSEAL (slider_size);
-
-  /* The upper limit of the ruler (in points) */
-  gdouble GSEAL (lower);
-  /* The lower limit of the ruler */
-  gdouble GSEAL (upper);
-  /* The position of the mark on the ruler */
-  gdouble GSEAL (position);
-  /* The maximum size of the ruler */
-  gdouble GSEAL (max_size);
+  /*< private >*/
+  GtkRulerPrivate *priv;
 };
 
 struct _GtkRulerClass
 {
   GtkWidgetClass parent_class;
 
-  void (* draw_ticks) (GtkRuler *ruler);
-  void (* draw_pos)   (GtkRuler *ruler);
+  void (* draw_ticks) (GtkRuler *ruler,
+                       cairo_t  *cr);
+  void (* draw_pos)   (GtkRuler *ruler,
+                       cairo_t  *cr);
 
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
@@ -110,6 +100,7 @@ struct _GtkRulerMetric
 
 
 GType           gtk_ruler_get_type   (void) G_GNUC_CONST;
+GtkWidget     * gtk_ruler_new        (GtkOrientation  orientation);
 void            gtk_ruler_set_metric (GtkRuler       *ruler,
                                       GtkMetricType   metric);
 GtkMetricType   gtk_ruler_get_metric (GtkRuler       *ruler);
@@ -124,9 +115,6 @@ void            gtk_ruler_get_range  (GtkRuler       *ruler,
                                       gdouble        *position,
                                       gdouble        *max_size);
 
-void            gtk_ruler_draw_ticks (GtkRuler       *ruler);
-void            gtk_ruler_draw_pos   (GtkRuler       *ruler);
-
 G_END_DECLS
 
 #endif /* __GTK_RULER_H__ */
