@@ -28,14 +28,16 @@
  */
 
 #include "config.h"
-#include <string.h>
 
 #include "gtkstatusicon.h"
 
+#include <string.h>
+
 #include "gtkintl.h"
 #include "gtkiconfactory.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
+#include "gtksizerequest.h"
 #include "gtktrayicon.h"
 
 #include "gtkprivate.h"
@@ -140,11 +142,6 @@ struct _GtkStatusIconPrivate
       GIcon     *gicon;
     } image_data;
 
-  GdkPixbuf    *blank_icon;
-  guint         blinking_timeout;
-
-  guint         blinking : 1;
-  guint         blink_off : 1;
   guint         visible : 1;
 };
 
@@ -189,7 +186,6 @@ static gboolean gtk_status_icon_button_press     (GtkStatusIcon  *status_icon,
 						  GdkEventButton *event);
 static gboolean gtk_status_icon_button_release   (GtkStatusIcon  *status_icon,
 						  GdkEventButton *event);
-static void     gtk_status_icon_disable_blinking (GtkStatusIcon  *status_icon);
 static void     gtk_status_icon_reset_image_data (GtkStatusIcon  *status_icon);
 static void     gtk_status_icon_update_image    (GtkStatusIcon *status_icon);
 
@@ -285,14 +281,6 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
 							GDK_TYPE_SCREEN,
  							GTK_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class,
-				   PROP_BLINKING,
-				   g_param_spec_boolean ("blinking",
-							 P_("Blinking"),
-							 P_("Whether the status icon is blinking"),
-							 FALSE,
-							 GTK_PARAM_READWRITE));
-
   g_object_class_install_property (gobject_class,
 				   PROP_VISIBLE,
 				   g_param_spec_boolean ("visible",
@@ -972,14 +960,8 @@ gtk_status_icon_finalize (GObject *object)
   GtkStatusIcon *status_icon = GTK_STATUS_ICON (object);
   GtkStatusIconPrivate *priv = status_icon->priv;
 
-  gtk_status_icon_disable_blinking (status_icon);
-  
   gtk_status_icon_reset_image_data (status_icon);
 
-  if (priv->blank_icon)
-    g_object_unref (priv->blank_icon);
-  priv->blank_icon = NULL;
-
 #ifdef GDK_WINDOWING_X11
   g_signal_handlers_disconnect_by_func (priv->tray_icon,
 			                gtk_status_icon_key_press, status_icon);
@@ -1059,9 +1041,6 @@ gtk_status_icon_set_property (GObject      *object,
     case PROP_SCREEN:
       gtk_status_icon_set_screen (status_icon, g_value_get_object (value));
       break;
-    case PROP_BLINKING:
-      gtk_status_icon_set_blinking (status_icon, g_value_get_boolean (value));
-      break;
     case PROP_VISIBLE:
       gtk_status_icon_set_visible (status_icon, g_value_get_boolean (value));
       break;
@@ -1133,9 +1112,6 @@ gtk_status_icon_get_property (GObject    *object,
     case PROP_SCREEN:
       g_value_set_object (value, gtk_status_icon_get_screen (status_icon));
       break;
-    case PROP_BLINKING:
-      g_value_set_boolean (value, gtk_status_icon_get_blinking (status_icon));
-      break;
     case PROP_VISIBLE:
       g_value_set_boolean (value, gtk_status_icon_get_visible (status_icon));
       break;
@@ -1321,37 +1297,6 @@ emit_size_changed_signal (GtkStatusIcon *status_icon,
 
 #endif
 
-static GdkPixbuf *
-gtk_status_icon_blank_icon (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-
-  if (priv->blank_icon)
-    {
-      gint width, height;
-
-      width  = gdk_pixbuf_get_width (priv->blank_icon);
-      height = gdk_pixbuf_get_height (priv->blank_icon);
-
-
-      if (width == priv->image_width && height == priv->image_height)
-	return priv->blank_icon;
-      else
-	{
-	  g_object_unref (priv->blank_icon);
-	  priv->blank_icon = NULL;
-	}
-    }
-
-  priv->blank_icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				     priv->image_width, 
-				     priv->image_height);
-  if (priv->blank_icon)
-    gdk_pixbuf_fill (priv->blank_icon, 0);
-
-  return priv->blank_icon;
-}
-
 #ifdef GDK_WINDOWING_X11
 
 static GtkIconSize
@@ -1400,30 +1345,6 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
   HICON prev_hicon;
 #endif
 
-  if (priv->blink_off)
-    {
-#ifdef GDK_WINDOWING_X11
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image),
-				 gtk_status_icon_blank_icon (status_icon));
-#endif
-#ifdef GDK_WINDOWING_WIN32
-      prev_hicon = priv->nid.hIcon;
-      priv->nid.hIcon = gdk_win32_pixbuf_to_hicon_libgtk_only (gtk_status_icon_blank_icon (status_icon));
-      priv->nid.uFlags |= NIF_ICON;
-      if (priv->nid.hWnd != NULL && priv->visible)
-	if (!Shell_NotifyIconW (NIM_MODIFY, &priv->nid))
-	  g_warning (G_STRLOC ": Shell_NotifyIcon(NIM_MODIFY) failed");
-      if (prev_hicon)
-	DestroyIcon (prev_hicon);
-#endif
-#ifdef GDK_WINDOWING_QUARTZ
-      QUARTZ_POOL_ALLOC;
-      [priv->status_item setImage:gtk_status_icon_blank_icon (status_icon)];
-      QUARTZ_POOL_RELEASE;
-#endif
-      return;
-    }
-
   switch (priv->storage_type)
     {
     case GTK_IMAGE_PIXBUF:
@@ -1783,11 +1704,11 @@ gtk_status_icon_key_press (GtkStatusIcon  *status_icon,
   state = event->state & gtk_accelerator_get_default_mod_mask ();
   keyval = event->keyval;
   if (state == 0 &&
-      (keyval == GDK_Return ||
-       keyval == GDK_KP_Enter ||
-       keyval == GDK_ISO_Enter ||
-       keyval == GDK_space ||
-       keyval == GDK_KP_Space))
+      (keyval == GDK_KEY_Return ||
+       keyval == GDK_KEY_KP_Enter ||
+       keyval == GDK_KEY_ISO_Enter ||
+       keyval == GDK_KEY_space ||
+       keyval == GDK_KEY_KP_Space))
     {
       emit_activate_signal (status_icon);
       return TRUE;
@@ -2103,7 +2024,8 @@ gtk_status_icon_get_storage_type (GtkStatusIcon *status_icon)
  * The caller of this function does not own a reference to the
  * returned pixbuf.
  * 
- * Return value: the displayed pixbuf, or %NULL if the image is empty.
+ * Return value: (transfer none): the displayed pixbuf,
+ *     or %NULL if the image is empty.
  *
  * Since: 2.10
  **/
@@ -2202,7 +2124,7 @@ gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
  *
  * If this function fails, @icon is left unchanged;
  *
- * Returns: the displayed icon, or %NULL if the image is empty
+ * Returns: (transfer none): the displayed icon, or %NULL if the image is empty
  *
  * Since: 2.14
  **/
@@ -2271,13 +2193,13 @@ gtk_status_icon_set_screen (GtkStatusIcon *status_icon,
 #endif
 }
 
-/** 
+/**
  * gtk_status_icon_get_screen:
  * @status_icon: a #GtkStatusIcon
  *
  * Returns the #GdkScreen associated with @status_icon.
  *
- * Return value: a #GdkScreen.
+ * Return value: (transfer none): a #GdkScreen.
  *
  * Since: 2.12
  */
@@ -2286,56 +2208,13 @@ gtk_status_icon_get_screen (GtkStatusIcon *status_icon)
 {
   g_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), NULL);
 
-#ifdef GDK_WINDOWING_X11   
+#ifdef GDK_WINDOWING_X11
   return gtk_window_get_screen (GTK_WINDOW (status_icon->priv->tray_icon));
 #else
   return gdk_screen_get_default ();
 #endif
 }
 
-static gboolean
-gtk_status_icon_blinker (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  priv->blink_off = !priv->blink_off;
-
-  gtk_status_icon_update_image (status_icon);
-
-  return TRUE;
-}
-
-static void
-gtk_status_icon_enable_blinking (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  if (!priv->blinking_timeout)
-    {
-      gtk_status_icon_blinker (status_icon);
-
-      priv->blinking_timeout =
-	gdk_threads_add_timeout (BLINK_TIMEOUT, 
-		       (GSourceFunc) gtk_status_icon_blinker, 
-		       status_icon);
-    }
-}
-
-static void
-gtk_status_icon_disable_blinking (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  if (priv->blinking_timeout)
-    {
-      g_source_remove (priv->blinking_timeout);
-      priv->blinking_timeout = 0;
-      priv->blink_off = FALSE;
-
-      gtk_status_icon_update_image (status_icon);
-    }
-}
-
 /**
  * gtk_status_icon_set_visible:
  * @status_icon: a #GtkStatusIcon
@@ -2409,62 +2288,6 @@ gtk_status_icon_get_visible (GtkStatusIcon *status_icon)
   return status_icon->priv->visible;
 }
 
-/**
- * gtk_status_icon_set_blinking:
- * @status_icon: a #GtkStatusIcon
- * @blinking: %TRUE to turn blinking on, %FALSE to turn it off
- * 
- * Makes the status icon start or stop blinking. 
- * Note that blinking user interface elements may be problematic
- * for some users, and thus may be turned off, in which case
- * this setting has no effect.
- *
- * Since: 2.10
- **/
-void
-gtk_status_icon_set_blinking (GtkStatusIcon *status_icon,
-			      gboolean       blinking)
-{
-  GtkStatusIconPrivate *priv;
-
-  g_return_if_fail (GTK_IS_STATUS_ICON (status_icon));
-
-  priv = status_icon->priv;
-
-  blinking = blinking != FALSE;
-
-  if (priv->blinking != blinking)
-    {
-      priv->blinking = blinking;
-
-      if (blinking)
-	gtk_status_icon_enable_blinking (status_icon);
-      else
-	gtk_status_icon_disable_blinking (status_icon);
-
-      g_object_notify (G_OBJECT (status_icon), "blinking");
-    }
-}
-
-/**
- * gtk_status_icon_get_blinking:
- * @status_icon: a #GtkStatusIcon
- * 
- * Returns whether the icon is blinking, see 
- * gtk_status_icon_set_blinking().
- * 
- * Return value: %TRUE if the icon is blinking
- *
- * Since: 2.10
- **/
-gboolean
-gtk_status_icon_get_blinking (GtkStatusIcon *status_icon)
-{
-  g_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), FALSE);
-
-  return status_icon->priv->blinking;
-}
-
 /**
  * gtk_status_icon_is_embedded:
  * @status_icon: a #GtkStatusIcon
@@ -2489,7 +2312,7 @@ gtk_status_icon_is_embedded (GtkStatusIcon *status_icon)
 #ifdef GDK_WINDOWING_X11
   plug = GTK_PLUG (status_icon->priv->tray_icon);
 
-  if (plug->socket_window)
+  if (gtk_plug_get_embedded (plug))
     return TRUE;
   else
     return FALSE;
@@ -2524,6 +2347,7 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
 			       gpointer  user_data)
 {
 #ifdef GDK_WINDOWING_X11
+  GtkAllocation allocation;
   GtkStatusIcon *status_icon;
   GtkStatusIconPrivate *priv;
   GtkTrayIcon *tray_icon;
@@ -2532,6 +2356,7 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
   GtkTextDirection direction;
   GtkRequisition menu_req;
   GdkRectangle monitor;
+  GdkWindow *window;
   gint monitor_num, height, width, xoffset, yoffset;
   
   g_return_if_fail (GTK_IS_MENU (menu));
@@ -2547,30 +2372,33 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
   screen = gtk_widget_get_screen (widget);
   gtk_menu_set_screen (menu, screen);
 
-  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+  window = gtk_widget_get_window (widget);
+  monitor_num = gdk_screen_get_monitor_at_window (screen, window);
   if (monitor_num < 0)
     monitor_num = 0;
   gtk_menu_set_monitor (menu, monitor_num);
 
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-  gdk_window_get_origin (widget->window, x, y);
-  
-  gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
+  gdk_window_get_origin (window, x, y);
 
+  gtk_widget_get_preferred_size (GTK_WIDGET (menu),
+                                 &menu_req, NULL);
+
+  gtk_widget_get_allocation (widget, &allocation);
   if (_gtk_tray_icon_get_orientation (tray_icon) == GTK_ORIENTATION_VERTICAL)
     {
       width = 0;
-      height = widget->allocation.height;
-      xoffset = widget->allocation.width;
+      height = allocation.height;
+      xoffset = allocation.width;
       yoffset = 0;
     }
   else
     {
-      width = widget->allocation.width;
+      width = allocation.width;
       height = 0;
       xoffset = 0;
-      yoffset = widget->allocation.height;
+      yoffset = allocation.height;
     }
 
   if (direction == GTK_TEXT_DIR_RTL)
@@ -2661,6 +2489,7 @@ gtk_status_icon_get_geometry (GtkStatusIcon    *status_icon,
 			      GtkOrientation   *orientation)
 {
 #ifdef GDK_WINDOWING_X11   
+  GtkAllocation allocation;
   GtkWidget *widget;
   GtkStatusIconPrivate *priv;
   gint x, y;
@@ -2675,11 +2504,14 @@ gtk_status_icon_get_geometry (GtkStatusIcon    *status_icon,
 
   if (area)
     {
-      gdk_window_get_origin (widget->window, &x, &y);
+      gdk_window_get_origin (gtk_widget_get_window (widget),
+                             &x, &y);
+
+      gtk_widget_get_allocation (widget, &allocation);
       area->x = x;
       area->y = y;
-      area->width = widget->allocation.width;
-      area->height = widget->allocation.height;
+      area->width = allocation.width;
+      area->height = allocation.height;
     }
 
   if (orientation)
@@ -2961,7 +2793,7 @@ gtk_status_icon_get_x11_window_id (GtkStatusIcon *status_icon)
 {
 #ifdef GDK_WINDOWING_X11
   gtk_widget_realize (GTK_WIDGET (status_icon->priv->tray_icon));
-  return GDK_WINDOW_XID (GTK_WIDGET (status_icon->priv->tray_icon)->window);
+  return GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (status_icon->priv->tray_icon)));
 #else
   return 0;
 #endif
