@@ -63,6 +63,8 @@ GType gtk_entry_icon_position_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ENTRY_ICON_POSITION (gtk_entry_icon_position_get_type ())
 
 /* enumerations from "gtkenums.h" */
+GType gtk_align_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_ALIGN (gtk_align_get_type ())
 GType gtk_arrow_placement_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ARROW_PLACEMENT (gtk_arrow_placement_get_type ())
 GType gtk_arrow_type_get_type (void) G_GNUC_CONST;
@@ -159,12 +161,8 @@ GType gtk_drag_result_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DRAG_RESULT (gtk_drag_result_get_type ())
 GType gtk_size_request_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SIZE_REQUEST_MODE (gtk_size_request_mode_get_type ())
-GType gtk_wrap_allocation_mode_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_WRAP_ALLOCATION_MODE (gtk_wrap_allocation_mode_get_type ())
-GType gtk_wrap_box_spreading_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_WRAP_BOX_SPREADING (gtk_wrap_box_spreading_get_type ())
-GType gtk_wrap_box_packing_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_WRAP_BOX_PACKING (gtk_wrap_box_packing_get_type ())
+GType gtk_scrollable_policy_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_SCROLLABLE_POLICY (gtk_scrollable_policy_get_type ())
 
 /* enumerations from "gtkfilechooser.h" */
 GType gtk_file_chooser_action_get_type (void) G_GNUC_CONST;
@@ -200,10 +198,6 @@ GType gtk_buttons_type_get_type (void) G_GNUC_CONST;
 GType gtk_notebook_tab_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_NOTEBOOK_TAB (gtk_notebook_tab_get_type ())
 
-/* enumerations from "gtkobject.h" */
-GType gtk_object_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_OBJECT_FLAGS (gtk_object_flags_get_type ())
-
 /* enumerations from "gtkprintoperation.h" */
 GType gtk_print_status_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRINT_STATUS (gtk_print_status_get_type ())
@@ -214,10 +208,6 @@ GType gtk_print_operation_action_get_type (void) G_GNUC_CONST;
 GType gtk_print_error_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRINT_ERROR (gtk_print_error_get_type ())
 
-/* enumerations from "gtkprivate.h" */
-GType gtk_private_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_PRIVATE_FLAGS (gtk_private_flags_get_type ())
-
 /* enumerations from "gtkrc.h" */
 GType gtk_rc_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RC_FLAGS (gtk_rc_flags_get_type ())
@@ -285,8 +275,6 @@ GType gtk_ui_manager_item_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_UI_MANAGER_ITEM_TYPE (gtk_ui_manager_item_type_get_type ())
 
 /* enumerations from "gtkwidget.h" */
-GType gtk_widget_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_WIDGET_FLAGS (gtk_widget_flags_get_type ())
 GType gtk_widget_help_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WIDGET_HELP_TYPE (gtk_widget_help_type_get_type ())
 G_END_DECLS
