@@ -43,6 +43,7 @@
 *tp++ = gdk_property_state_get_type();
 *tp++ = gdk_prop_mode_get_type();
 *tp++ = gdk_rectangle_get_type();
+*tp++ = gdk_rgba_get_type();
 *tp++ = gdk_screen_get_type();
 *tp++ = gdk_scroll_direction_get_type();
 *tp++ = gdk_setting_action_get_type();
@@ -147,6 +148,7 @@
 *tp++ = gtk_font_selection_dialog_get_type();
 *tp++ = gtk_font_selection_get_type();
 *tp++ = gtk_frame_get_type();
+*tp++ = gtk_grid_get_type();
 *tp++ = gtk_handle_box_get_type();
 *tp++ = gtk_hbox_get_type();
 *tp++ = gtk_hbutton_box_get_type();
@@ -252,6 +254,8 @@
 *tp++ = gtk_ruler_get_type();
 *tp++ = gtk_scale_button_get_type();
 *tp++ = gtk_scale_get_type();
+*tp++ = gtk_scrollable_get_type();
+*tp++ = gtk_scrollable_policy_get_type();
 *tp++ = gtk_scrollbar_get_type();
 *tp++ = gtk_scrolled_window_get_type();
 *tp++ = gtk_scroll_step_get_type();
