@@ -32,6 +32,7 @@
 #define __GTK_WINDOW_H__
 
 
+#include <gtk/gtkapplication.h>
 #include <gtk/gtkaccelgroup.h>
 #include <gtk/gtkbin.h>
 
@@ -74,15 +75,8 @@ struct _GtkWindowClass
   void     (* activate_focus)          (GtkWindow       *window);
   void     (* activate_default)        (GtkWindow       *window);
 
-  /* as of GTK+ 2.12 the "move-focus" signal has been moved to GtkWidget,
-   * so this is merley a virtual function now. Overriding it in subclasses
-   * continues to work though.
-   */
-  void     (* move_focus)              (GtkWindow       *window,
-                                        GtkDirectionType direction);
-  
-  void	   (*keys_changed)	       (GtkWindow	*window);
-  
+  void	   (* keys_changed)	       (GtkWindow	*window);
+
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
@@ -345,6 +339,10 @@ GList *          gtk_window_group_list_windows  (GtkWindowGroup     *window_grou
 GtkWidget *      gtk_window_group_get_current_device_grab (GtkWindowGroup *window_group,
                                                            GdkDevice      *device);
 
+GtkApplication *gtk_window_get_application      (GtkWindow          *window);
+void            gtk_window_set_application      (GtkWindow          *window,
+                                                 GtkApplication     *application);
+
 
 /* Window grips
  */
