@@ -352,10 +352,10 @@ test_uimanager_simple (void)
   children = gtk_container_get_children (GTK_CONTAINER (menubar));
   menu = children->data;
   g_assert (GTK_IS_MENU_ITEM (menu));
-  g_assert (strcmp (GTK_WIDGET (menu)->name, "file") == 0);
+  g_assert (strcmp (gtk_widget_get_name (GTK_WIDGET (menu)), "file") == 0);
   g_list_free (children);
   
-  label = G_OBJECT (GTK_BIN (menu)->child);
+  label = G_OBJECT (gtk_bin_get_child (GTK_BIN (menu)));
   g_assert (GTK_IS_LABEL (label));
   g_assert (strcmp (gtk_label_get_text (GTK_LABEL (label)), "File") == 0);
 
@@ -970,6 +970,7 @@ static void
 test_children (void)
 {
   GtkBuilder * builder;
+  GtkWidget *content_area, *dialog_action_area;
   GList *children;
   const gchar buffer1[] =
     "<interface>"
@@ -1008,8 +1009,8 @@ test_children (void)
   button = gtk_builder_get_object (builder, "button1");
   g_assert (button != NULL);
   g_assert (GTK_IS_BUTTON (button));
-  g_assert (GTK_WIDGET(button)->parent != NULL);
-  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (GTK_WIDGET (button)->parent)), "window1") == 0);
+  g_assert (gtk_widget_get_parent (GTK_WIDGET(button)) != NULL);
+  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (gtk_widget_get_parent (GTK_WIDGET (button)))), "window1") == 0);
 
   gtk_widget_destroy (GTK_WIDGET (window));
   g_object_unref (builder);
@@ -1023,20 +1024,24 @@ test_children (void)
   g_list_free (children);
   
   vbox = gtk_builder_get_object (builder, "dialog1-vbox");
+  content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
   g_assert (vbox != NULL);
-  g_assert (GTK_IS_VBOX (vbox));
-  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (GTK_WIDGET (vbox)->parent)), "dialog1") == 0);
-  g_assert (GTK_CONTAINER (vbox)->border_width == 10);
-  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (GTK_DIALOG (dialog)->vbox)), "dialog1-vbox") == 0);
+  g_assert (GTK_IS_BOX (vbox));
+  g_assert (gtk_orientable_get_orientation (GTK_ORIENTABLE (vbox)) == GTK_ORIENTATION_VERTICAL);
+  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (gtk_widget_get_parent (GTK_WIDGET (vbox)))), "dialog1") == 0);
+  g_assert (gtk_container_get_border_width (GTK_CONTAINER (vbox)) == 10);
+  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (content_area)), "dialog1-vbox") == 0);
 
   action_area = gtk_builder_get_object (builder, "dialog1-action_area");
+  dialog_action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
   g_assert (action_area != NULL);
-  g_assert (GTK_IS_HBUTTON_BOX (action_area));
-  g_assert (GTK_WIDGET (action_area)->parent != NULL);
-  g_assert (GTK_CONTAINER (action_area)->border_width == 20);
-  g_assert (GTK_DIALOG (dialog)->action_area != NULL);
-  g_assert (gtk_buildable_get_name (GTK_BUILDABLE (GTK_DIALOG (dialog)->action_area)) != NULL);
-  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (GTK_DIALOG (dialog)->action_area)), "dialog1-action_area") == 0);
+  g_assert (GTK_IS_BUTTON_BOX (action_area));
+  g_assert (gtk_orientable_get_orientation (GTK_ORIENTABLE (action_area)) == GTK_ORIENTATION_HORIZONTAL);
+  g_assert (gtk_widget_get_parent (GTK_WIDGET (action_area)) != NULL);
+  g_assert (gtk_container_get_border_width (GTK_CONTAINER (action_area)) == 20);
+  g_assert (dialog_action_area != NULL);
+  g_assert (gtk_buildable_get_name (GTK_BUILDABLE (action_area)) != NULL);
+  g_assert (strcmp (gtk_buildable_get_name (GTK_BUILDABLE (dialog_action_area)), "dialog1-action_area") == 0);
   gtk_widget_destroy (GTK_WIDGET (dialog));
   g_object_unref (builder);
 }
@@ -1333,8 +1338,9 @@ test_combo_box_entry (void)
     "  </object>"
     "  <object class=\"GtkWindow\" id=\"window1\">"
     "    <child>"
-    "      <object class=\"GtkComboBoxEntry\" id=\"comboboxentry1\">"
+    "      <object class=\"GtkComboBox\" id=\"comboboxentry1\">"
     "        <property name=\"model\">liststore1</property>"
+    "        <property name=\"has-entry\">True</property>"
     "        <property name=\"visible\">True</property>"
     "        <child>"
     "          <object class=\"GtkCellRendererText\" id=\"renderer1\"/>"
@@ -1493,6 +1499,36 @@ test_dialog (void)
   g_object_unref (builder);
 }
 
+static void
+test_message_dialog (void)
+{
+  GtkBuilder * builder;
+  const gchar buffer1[] =
+    "<interface>"
+    "  <object class=\"GtkMessageDialog\" id=\"dialog1\">"
+    "    <child internal-child=\"message_area\">"
+    "      <object class=\"GtkVBox\" id=\"dialog-message-area\">"
+    "        <child>"
+    "          <object class=\"GtkExpander\" id=\"expander\"/>"
+    "        </child>"
+    "      </object>"
+    "    </child>"
+    "  </object>"
+    "</interface>";
+
+  GObject *dialog1;
+  GObject *expander;
+
+  builder = builder_new_from_string (buffer1, -1, NULL);
+  dialog1 = gtk_builder_get_object (builder, "dialog1");
+  expander = gtk_builder_get_object (builder, "expander");
+  g_assert (GTK_IS_EXPANDER (expander));
+  g_assert (gtk_widget_get_parent (GTK_WIDGET (expander)) == gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog1)));
+
+  gtk_widget_destroy (GTK_WIDGET (dialog1));
+  g_object_unref (builder);
+}
+
 static void
 test_accelerators (void)
 {
@@ -1811,17 +1847,7 @@ test_value_from_string (void)
                              GTK_BUILDER_ERROR_INVALID_VALUE));
   g_error_free (error);
   error = NULL;
-  
-  g_assert (gtk_builder_value_from_string_type (builder, GTK_TYPE_WIDGET_FLAGS, "mapped", &value, &error) == TRUE);
-  g_assert (G_VALUE_HOLDS_FLAGS (&value));
-  g_assert (g_value_get_flags (&value) == GTK_MAPPED);
-  g_value_unset (&value);
 
-  g_assert (gtk_builder_value_from_string_type (builder, GTK_TYPE_WIDGET_FLAGS, "GTK_VISIBLE | GTK_REALIZED", &value, &error) == TRUE);
-  g_assert (G_VALUE_HOLDS_FLAGS (&value));
-  g_assert (g_value_get_flags (&value) == (GTK_VISIBLE | GTK_REALIZED));
-  g_value_unset (&value);
-  
   g_assert (gtk_builder_value_from_string_type (builder, GTK_TYPE_WINDOW_TYPE, "foobar", &value, &error) == FALSE);
   g_value_unset (&value);
   g_assert (g_error_matches (error,
@@ -2259,10 +2285,10 @@ test_add_objects (void)
   menu = children->data;
   g_assert (menu != NULL);
   g_assert (GTK_IS_MENU_ITEM (menu));
-  g_assert (strcmp (GTK_WIDGET (menu)->name, "file") == 0);
+  g_assert (strcmp (gtk_widget_get_name (GTK_WIDGET (menu)), "file") == 0);
   g_list_free (children);
  
-  label = G_OBJECT (GTK_BIN (menu)->child);
+  label = G_OBJECT (gtk_bin_get_child (GTK_BIN (menu)));
   g_assert (label != NULL);
   g_assert (GTK_IS_LABEL (label));
   g_assert (strcmp (gtk_label_get_text (GTK_LABEL (label)), "File") == 0);
@@ -2286,10 +2312,10 @@ test_add_objects (void)
   menu = children->data;
   g_assert (menu != NULL);
   g_assert (GTK_IS_MENU_ITEM (menu));
-  g_assert (strcmp (GTK_WIDGET (menu)->name, "file") == 0);
+  g_assert (strcmp (gtk_widget_get_name (GTK_WIDGET (menu)), "file") == 0);
   g_list_free (children);
  
-  label = G_OBJECT (GTK_BIN (menu)->child);
+  label = G_OBJECT (gtk_bin_get_child (GTK_BIN (menu)));
   g_assert (label != NULL);
   g_assert (GTK_IS_LABEL (label));
   g_assert (strcmp (gtk_label_get_text (GTK_LABEL (label)), "File") == 0);
@@ -2300,16 +2326,18 @@ test_add_objects (void)
 static GtkWidget *
 get_parent_menubar (GtkWidget *menuitem)
 {
-  GtkMenuShell *menu_shell = (GtkMenuShell *)menuitem->parent;
+  GtkMenuShell *menu_shell;
   GtkWidget *attach = NULL;
 
+  menu_shell = GTK_MENU_SHELL (gtk_widget_get_parent (menuitem));
+
   g_assert (GTK_IS_MENU_SHELL (menu_shell));
 
   while (menu_shell && !GTK_IS_MENU_BAR (menu_shell))
     {
       if (GTK_IS_MENU (menu_shell) && 
 	  (attach = gtk_menu_get_attach_widget (GTK_MENU (menu_shell))) != NULL)
-	menu_shell = (GtkMenuShell *)attach->parent;
+	menu_shell = GTK_MENU_SHELL (gtk_widget_get_parent (attach));
       else
 	menu_shell = NULL;
     }
@@ -2393,6 +2421,7 @@ test_menus (void)
     "<object class=\"GtkAccelGroup\" id=\"accelgroup1\"/>"
     "</interface>";
   GtkBuilder *builder;
+  GtkWidget *child;
   GtkWidget *window, *item;
   GtkAccelGroup *accel_group;
   GtkWidget *item_accel_label, *sample_accel_label, *sample_menu_item, *custom;
@@ -2408,14 +2437,16 @@ test_menus (void)
 
   sample_menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_NEW, accel_group);
 
-  g_assert (GTK_BIN (sample_menu_item)->child);
-  g_assert (GTK_IS_ACCEL_LABEL (GTK_BIN (sample_menu_item)->child));
-  sample_accel_label = GTK_WIDGET (GTK_BIN (sample_menu_item)->child);
+  child = gtk_bin_get_child (GTK_BIN (sample_menu_item));
+  g_assert (child);
+  g_assert (GTK_IS_ACCEL_LABEL (child));
+  sample_accel_label = child;
   gtk_widget_show (sample_accel_label);
 
-  g_assert (GTK_BIN (item)->child);
-  g_assert (GTK_IS_ACCEL_LABEL (GTK_BIN (item)->child));
-  item_accel_label = GTK_WIDGET (GTK_BIN (item)->child);
+  child = gtk_bin_get_child (GTK_BIN (item));
+  g_assert (child);
+  g_assert (GTK_IS_ACCEL_LABEL (child));
+  item_accel_label = child;
 
   gtk_accel_label_refetch (GTK_ACCEL_LABEL (sample_accel_label));
   gtk_accel_label_refetch (GTK_ACCEL_LABEL (item_accel_label));
@@ -2441,7 +2472,7 @@ test_menus (void)
   item = (GtkWidget *)gtk_builder_get_object (builder, "imagemenuitem1");
   custom = (GtkWidget *)gtk_builder_get_object (builder, "custom1");
 
-  g_assert (custom->parent == item);
+  g_assert (gtk_widget_get_parent (custom) == item);
 
   gtk_widget_destroy (GTK_WIDGET (window));
   g_object_unref (builder);
@@ -2586,6 +2617,7 @@ main (int argc, char **argv)
   g_test_add_func ("/Builder/AddObjects", test_add_objects);
   g_test_add_func ("/Builder/Menus", test_menus);
   g_test_add_func ("/Builder/MessageArea", test_message_area);
+  g_test_add_func ("/Builder/MessageDialog", test_message_dialog);
 
   return g_test_run();
 }
