@@ -109,7 +109,7 @@ static GtkPageSetup *       file_printer_get_default_page_size     (GtkPrinter
 static void
 gtk_print_backend_file_register_type (GTypeModule *module)
 {
-  static const GTypeInfo print_backend_file_info =
+  const GTypeInfo print_backend_file_info =
   {
     sizeof (GtkPrintBackendFileClass),
     NULL,		/* base_init */
@@ -496,7 +496,7 @@ gtk_print_backend_file_init (GtkPrintBackendFile *backend)
 			  NULL); 
 
   gtk_printer_set_has_details (printer, TRUE);
-  gtk_printer_set_icon_name (printer, "gtk-save");
+  gtk_printer_set_icon_name (printer, "document-save");
   gtk_printer_set_is_active (printer, TRUE);
 
   gtk_print_backend_add_printer (GTK_PRINT_BACKEND (backend), printer);
